from starlite import Request
from starlite.response import Response
from starlite.types import ASGIApp, Message, Receive, Scope, Send

from starlite_starception.exception_handler import exception_handler


class StarceptionMiddleware:
    """
    Handles returning 500 responses when a server error occurs.

    If 'debug' is set, then traceback responses will be returned,
    otherwise the designated 'handler' will be called.

    This middleware class should generally be used to wrap *everything*
    else up, so that unhandled exceptions anywhere in the stack
    always result in an appropriate 500 response.
    """

    def __init__(self, app: ASGIApp) -> None:
        self.app = app

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        if scope["type"] != "http":  # pragma: nocover
            await self.app(scope, receive, send)
            return

        response_started = False

        async def _send(message: Message) -> None:
            nonlocal response_started, send

            if message["type"] == "http.response.start":  # pragma: nocover
                response_started = True
            await send(message)

        try:
            await self.app(scope, receive, _send)
        except Exception as exc:
            if not response_started:
                request = Request(scope)
                if request.app.debug:
                    # In debug mode, return traceback responses.
                    response = self.debug_response(request, exc)
                else:
                    # Use our default 500 error handler.
                    response = self.error_response(request, exc)

                await response(scope, receive, send)

            # We always continue to raise the exception.
            # This allows servers to log the error, or allows test clients
            # to optionally raise the error within the test case.
            raise exc

    def debug_response(self, request: Request, exc: Exception) -> Response:
        return exception_handler(request, exc)

    def error_response(self, request: Request, exc: Exception) -> Response:
        return Response("Internal Server Error", status_code=500, media_type="text/plain")



import dataclasses

import hashlib
import html
import inspect
import jinja2
import markupsafe
import os
import sys
import traceback
import typing
from markupsafe import Markup
from pprint import pformat
from starlite import Request
from starlite.datastructures import URL
from starlite.middleware.exceptions import ExceptionHandlerMiddleware
from starlite.response import Response
from urllib.parse import quote_plus

_editor: str = 'none'
open_link_templates: typing.Dict[str, str] = {
    'none': 'file://{path}',
    'vscode': 'vscode://file/{path}:{lineno}',
}


def set_editor(name: str) -> None:
    """
    Set code editor.

    We will use it to generate file open links. Built-in editors: vscode.
    """
    global _editor
    _editor = name


def add_link_template(editor: str, template: str) -> None:
    """
    Add open file link template. The template accepts two format keys: path and
    lineno.

    Example:
        add_link_template('vscode', 'vscode://file/{path}:{lineno}')
    """
    open_link_templates[editor] = template


def to_ide_link(path: str, lineno: int) -> str:
    """Generate open file link for current editor."""
    template = open_link_templates.get(_editor, open_link_templates['none'])
    return template.format(path=path, lineno=lineno)


def install_error_handler(editor: str = '') -> None:
    """
    Replace Starlette debug exception handler in-place.

    May be, someday, we won't need it.
    See https://github.com/encode/starlette/discussions/1867
    """
    set_editor(editor)

    def bound_handler(self: ExceptionHandlerMiddleware, request: Request, exc: Exception) -> Response:
        return exception_handler(request, exc)

    setattr(ExceptionHandlerMiddleware, 'default_http_exception_handler', bound_handler)


def get_relative_filename(path: str) -> str:
    for sys_path in reversed(sorted(sys.path)):
        if sys_path in path:
            path = path.replace(sys_path + "/", "")
            break
    return path


def format_qual_name(obj: typing.Any) -> str:
    if inspect.isclass(obj):
        module_name = obj.__module__
        type_name = obj.__name__
    else:
        module_name = obj.__class__.__module__
        type_name = obj.__class__.__name__

    if module_name == "builtins":
        module_name = ""
    if module_name:
        return f"{module_name}.{type_name}"
    return type_name


def frame_id(frame: inspect.FrameInfo) -> str:
    hash = hashlib.md5()
    hash.update(f"{frame.filename}{frame.lineno}".encode())
    return hash.hexdigest()


def is_vendor(frame: inspect.FrameInfo) -> bool:
    return sys.exec_prefix in frame.filename.replace('./', '')


def get_package_name(frame: inspect.FrameInfo) -> str:
    return typing.cast(str, frame.frame.f_globals.get("__package__", "").split(".")[0])


def get_symbol(frame: inspect.FrameInfo) -> str:
    symbol = ""
    if "self" in frame.frame.f_locals:
        try:
            symbol = type(frame.frame.f_locals["self"]).__name__
        except Exception:
            return 'n/a'

    if "cls" in frame.frame.f_locals and frame.frame.f_locals["cls"]:
        symbol = frame.frame.f_locals["cls"].__name__

    # if we cannot detect class name then just a method name will be rendered
    function = frame.function
    if symbol:
        function = symbol + "." + function
    return function


def get_package_dir(frame: inspect.FrameInfo) -> str:
    return frame.filename.replace(get_relative_filename(frame.filename), "")


def format_variable(var_value: typing.Any) -> str:
    try:
        return pformat(var_value, indent=2)
    except Exception as ex:
        return repr(ex)


def _hide_url_secrets(value: str) -> str:
    url = URL(value)
    if url.password:
        url = url.replace(password='*' * 8)
    return str(url)


def mask_secrets(value: str, key: str) -> str:
    key = key.lower()
    if key.endswith('_url'):
        return _hide_url_secrets(value)

    if any(
        [
            "key" in key,
            "token" in key,
            "password" in key,
            "secret" in key,
        ]
    ):
        return Markup(
            '<span data-reveal="{value}" style="cursor: pointer">{stars} <i>(click to reveal)</i></span> '.format(
                stars="*" * 8, value=value
            )
        )
    return value


def highlight(value: str, filename: str) -> str:
    try:
        from pygments import highlight
        from pygments.formatters import HtmlFormatter
        from pygments.lexers import CssLexer, HtmlLexer, JavascriptLexer, PythonLexer

        *_, extension = os.path.splitext(filename)
        mapping = {
            '.py': PythonLexer(),
            '.htm': HtmlLexer(),
            '.html': HtmlLexer(),
            '.css': CssLexer(),
            '.js': JavascriptLexer(),
        }
        lexer = mapping.get(extension)
        if lexer:
            return highlight(value, lexer, HtmlFormatter(nowrap=True))  # type: ignore
        return value
    except ImportError:
        return markupsafe.escape(value)


jinja = jinja2.Environment(loader=jinja2.PackageLoader(__name__))
jinja.filters.update(
    {
        'symbol': get_symbol,
        'frame_id': frame_id,
        'is_vendor': is_vendor,
        'highlight': highlight,
        'to_ide_link': to_ide_link,
        'mask_secrets': mask_secrets,
        'package_dir': get_package_dir,
        'package_name': get_package_name,
        'format_variable': format_variable,
        'relative_filename': get_relative_filename,
    }
)


def exception_handler(request: Request, exc: Exception) -> Response:
    accept = request.headers.get("accept", "")

    if "text/html" in accept:
        content = generate_html(request, exc)
        return Response(content, status_code=500, media_type="text/html")

    content = generate_plain_text(exc)
    return Response(content, status_code=500, media_type="text/plain")


def generate_plain_text(exc: Exception) -> str:
    return "".join(traceback.format_exception(type(exc), exc, exc.__traceback__))


@dataclasses.dataclass
class StackItem:
    exc: Exception
    frames: typing.List[inspect.FrameInfo]
    has_vendor_frames: bool
    solution: str


def generate_html(request: Request, exc: Exception, limit: int = 15) -> str:
    traceback_obj = traceback.TracebackException.from_exception(exc, capture_locals=True)
    frames = inspect.getinnerframes(exc.__traceback__, limit) if exc.__traceback__ else []
    stack = [
        StackItem(
            exc=exc,
            solution=getattr(exc, 'solution', ''),
            frames=frames,
            has_vendor_frames=any(is_vendor(f) for f in frames),
        )
    ]
    exception = exc
    cause = getattr(exception, '__cause__')
    while cause:
        frames = inspect.getinnerframes(cause.__traceback__, limit) if cause.__traceback__ else []
        stack.append(
            StackItem(
                exc=cause,
                solution=getattr(cause, 'solution', ''),
                frames=frames,
                has_vendor_frames=any(is_vendor(f) for f in frames),
            )
        )
        cause = getattr(cause, '__cause__')

    template = jinja.get_template('index.html')
    return template.render(
        {
            'search_query': quote_plus(f'{format_qual_name(traceback_obj.exc_type)} {str(exc)}'),
            'exception_class': format_qual_name(traceback_obj.exc_type),
            'error_message': str(exc) or '""',
            'stack': stack,
            'request_method': request.method,
            'request_path': request.url.path,
            'request_info': {
                "Method": request.method,
                "Path": request.url.path,
                "Path params": Markup(
                    "<br>".join(
                        [
                            f'<span class="text-muted">{k}</span> = {html.escape(mask_secrets(str(v), k))}'
                            for k, v in request.path_params.items()
                        ]
                    )
                ),
                "Query params": Markup(
                    "<br>".join(
                        [
                            f'<span class="text-muted">{k}</span> = {html.escape(mask_secrets(str(v), k))}'
                            for k, v in request.query_params.items()
                        ]
                    )
                ),
                "Content type": request.headers.get("Content-Type", ""),
                "Client": f"{request.client.host}:{request.client.port}" if request.client else 'unknown',
            },
            'request_headers': request.headers,
            'request_state': {k: v for k, v in request.state._state.items()},
            'session': _get_session_info(request),
            'cookies': request.cookies,
            'app_state': {k: v for k, v in request.app.state._state.items()} if 'app' in request.scope else {},
            'platform': {
                "Python version": sys.version,
                "Platform": sys.platform,
                "Python": sys.executable,
                "Paths": Markup("<br>".join(sys.path)),
            },
            'environment': os.environ,
            'solution': getattr(exc, 'solution', None),
        }
    )


def _get_session_info(request: Request) -> typing.Dict[str, typing.Any]:
    try:
        return dict(request.session if 'session' in request.scope else {})
    except Exception:
        return {}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['starlite_starception']

package_data = \
{'': ['*'], 'starlite_starception': ['templates/*']}

install_requires = \
['Jinja2>=3,<4', 'MarkupSafe>=2,<3', 'starlite', 'typing_extensions>=4.4,<5.0']

setup_kwargs = {
    'name': 'starlite-starception',
    'version': '1.0.1.1',
    'description': 'Beautiful debugging page for starlite apps.',
    'long_description': '##############\n\n# NOTE: this is a fork of https://github.com/alex-oleshkevich/starception\n\nAll rights to him and the contributors for that repo.\n\nThis fork exists to implement starception for starlite: https://github.com/starlite-api/starlite.\n\nStarlite left starlette since version [1.39.0] for there own implementations. \n\nTherefore the orginal repo doens\'t work\nanymore\n\nSeeing starception was originally built for starlette backed frameworks.\nSo a fork was needed to make this work with\nstarlite\n\nNote2 : I will keep the fork in sync with the main repo except for the starlette stuff.\n\n##############\n\n# Starception-starlite\n\nBeautiful exception page for Starlette and FastAPI apps.\n\n![PyPI](https://img.shields.io/pypi/v/starception)\n![GitHub Workflow Status](https://img.shields.io/github/workflow/status/alex-oleshkevich/starception/Lint%20and%20test)\n![GitHub](https://img.shields.io/github/license/alex-oleshkevich/starception)\n![Libraries.io dependency status for latest release](https://img.shields.io/librariesio/release/pypi/starception)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/starception)\n![GitHub Release Date](https://img.shields.io/github/release-date/alex-oleshkevich/starception)\n\n## Installation\n\nInstall `starlite-starception` using PIP or poetry:\n\n```bash\npip install starlite_starception\n# or\npoetry add starlite_starception\n```\n\n### With syntax highlight support\n\nIf you want to colorize code snippets, install `pygments` library.\n\n```bash\npip install starlite_starception[pygments]\n# or\npoetry add starlite_starception -E pygments\n```\n\n## Screenshot\n\n![image](screenshot.png)\n\n<details>\n<summary>Dark theme</summary>\n<div>\n    <img src="./dark.png">\n</div>\n</details>\n\n## Features\n\n* secrets masking\n* solution hints\n* code snippets\n* display request info: query, body, headers, cookies\n* session contents\n* request and app state\n* platform information\n* environment variables\n* syntax highlight\n* open paths in editor (vscode only)\n* exception chains\n* dark theme\n\nThe middleware will automatically mask any value which key contains `key`, `secret`, `token`, `password`.\n\n## Quick start\n\nSee example application in [examples/](examples/) directory of this repository.\n\n## Usage\n\nStarception will work only in debug mode so don\'t forget to set `debug=True` for local development.\n\n### Monkey patching Starlette\n\nTo replace built-in debug exception handler call `install_error_handler` before you create Starlette instance.\n> Currently, this is a recommended approach.\n\n```python\nfrom starlite_starception import install_error_handler\nfrom starlite import Starlite, get\n\n\n@get("/")\ndef view() -> None:\n    raise TypeError(\'Oops\')\n\n\ninstall_error_handler()\napp = Starlite(route_handlers=[view])\n```\n\n### Using middleware\n\nTo render a beautiful exception page you need to install a `StarceptionMiddleware` middleware to your application.\n\n\n> Note, to catch as many exceptions as possible the middleware has to be the first one in the stack.\n\n```python\nimport typing\n\nfrom starlite import Starlite\n\nfrom starlite_starception import StarceptionMiddleware\n\n\nasync def index_view() -> None:\n    raise Exception(\'Oops, something really went wrong...\')\n\n\napp = Starlite(\n    debug=True,\n    route_handlers=[index_view],\n    middleware=[\n        StarceptionMiddleware,\n        # other middleware go here\n    ]\n)\n```\n\n## Solution hints\n\nIf exception class has `solution` attribute then its content will be used as a solution hint.\n\n```python\nclass WithHintError(Exception):\n    solution = (\n        \'The connection to the database cannot be established. \'\n        \'Either the database server is down or connection credentials are invalid.\'\n    )\n```\n\n![image](hints.png)\n\n## Opening files in editor\n\nSet your current editor to open paths in your editor/IDE.\n\n```python\nfrom starlite_starception import set_editor\n\nset_editor(\'vscode\')\n```\n\n![image](link.png)\n\n\n> Note, currently only VSCode supported. If you know how to integrate other editors - please PR\n\n### Registering link templates\n\nIf your editor is not supported, you can add it by calling `add_link_template` and then selecting it with `set_editor`.\n\n```python\nfrom starlite_starception import set_editor, add_link_template\n\nadd_link_template(\'vscode\', \'vscode://file/{path}:{lineno}\')\nset_editor(\'vscode\')\n```\n\n## Credentials\n\n* Look and feel inspired by [Phoenix Framework](https://www.phoenixframework.org/).\n* Icons by [Tabler Icons](https://tabler-icons.io/).\n',
    'author': 'Alex Oleshkevich',
    'author_email': 'alex.oleshkevich@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/nielsvanhooy/starlite-starception',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from starlite_starception.exception_handler import (
    add_link_template,
    exception_handler,
    install_error_handler,
    set_editor,
)
from starlite_starception.middleware import StarceptionMiddleware

__all__ = [
    'StarceptionMiddleware',
    'exception_handler',
    'set_editor',
    'add_link_template',
    'install_error_handler',
]
